/* libiir/src/libiir/500_response.c
 * 
 *  Copyright: ©2011, Laurence Withers.
 *  Author: Laurence Withers <l@lwithers.me.uk>
 *  License: GPLv3
*/



static double complex
iir_response_aux(int nc, const double* c, int nd, const double* d, double freq)
{
    int i;
    double complex sum, resp, N;

    /* Z transform; substitutions:
     *
     * z = exp(jω)
     *   where j is the imaginary unit (written as I in the C code below)
     *   and where ω = 2πf
     *
     * z^-n = exp(jωn)
     *
     * exp(jx) = cos(x) + j.sin(x)
     *
     * z^-n = cos(-ωn) + j.sin(-ωn)
     *
     * N = -ω = -2πf
     */
    N = -2 * M_PI * freq;

    sum = 0;
    for(i = 0; i < nc; ++i) {
        sum += c[i] * (cos(i*N) + I * sin(i*N));
    }

    resp = sum;
    sum = 1;
    for(i = 1; i <= nd; ++i) {
        sum += d[i - 1] * (cos(i*N) + I * sin(i*N));
    }

    return resp / sum;
}



double complex
iir_response_c(const struct iir_coeff_t* coeff, double freq)
{
    return iir_response_aux(coeff->nc, coeff->c, coeff->nd, coeff->d, freq);
}



double complex
iir_response(const struct iir_filter_t* fi, double freq)
{
    double complex resp;

    for(resp = 1.0; fi; fi = fi->next) {
        resp *= iir_response_aux(fi->nc, fi->c, fi->nd, fi->d, freq);
    }
    return resp;
}



/* options for text editors
vim: expandtab:ts=4:sw=4
*/
